@@ -64,7 +64,7 @@
 
             negotiate_socks_connection/2% +DesiredEndpoint, +StreamPair
           ]).
-:- autoload(library(debug), [assertion/1, debug/3]).
+:- use_module(library(debug), [assertion/1, debug/3]).
 :- autoload(library(lists), [last/2, member/2, append/3, append/2]).
 :- autoload(library(apply), [maplist/3, maplist/2]).
 :- autoload(library(error),
@@ -93,7 +93,7 @@ simple as opening a file.  See also http_open/3.
 ==
 dump_swi_homepage :-
     setup_call_cleanup(
-        tcp_connect(www.swi-prolog.org:http, Stream, []),
+        tcp_connect('www.swi-prolog.org':http, Stream, []),
         ( format(Stream,
                  'GET / HTTP/1.1~n\c
                   Host: www.swi-prolog.org~n\c
@@ -101,7 +101,7 @@ dump_swi_homepage :-
           flush_output(Stream),
           copy_stream_data(Stream, current_output)
         ),
-        close(S)).
+        close(Stream)).
 ==
 
 To   deal   with   timeouts   and     multiple   connections,   threads,
