@@ -82,7 +82,8 @@ cert_file(File, Abs) :-
 
 :- begin_tests(https_open, [condition(run_network_tests)]).
 
-test(readme, Title == "# SWI-Prolog SSL interface") :-
+test(readme, [Title == "# SWI-Prolog SSL interface",
+              timeout(20)]) :-
     http_download('https://raw.githubusercontent.com\c
 		      /SWI-Prolog/packages-ssl/master/README.md',
 		  String),
@@ -302,7 +303,8 @@ make_server(SSL, Socket) :-
 		  certificate_file('tests/test_certs/server-cert.pem'),
 		  key_file('tests/test_certs/server-key.pem'),
 		  cert_verify_hook(get_cert_verify),
-		  pem_password_hook(get_server_pwd)
+		  pem_password_hook(get_server_pwd),
+                  close_parent(true)
 		]),
     tcp_socket(Socket),
     tcp_setopt(Socket, reuseaddr),
@@ -740,6 +742,18 @@ test(validate_password) :-
 			 JCdZjj3ZLqgl1Yj5jRvDoCIP+FL0A13iem6eXfonBUvs0g6U6fAd8R9PTLQaq\c
 			 sUVawoj2S2z1lPPAMd3EfYVvw').
 
+test(validate_password_bcrypt) :-
+    crypto_password_hash("foobar",
+                         '$2a$11$pqt9BvPb4wOjkj5c9F14ge5R8HuI2i07hMXXk.IWciB370d2EXwX2').
+
+test(validate_password_bcrypt_roundtrip) :-
+    crypto_password_hash("hello", Hash, [algorithm(bcrypt)]),
+    crypto_password_hash("hello", Hash).
+
+test(validate_password_bcrypt_negative, [ fail ]) :-
+    crypto_password_hash("hello",
+                         '$2a$11$pqt9BvPb4wOjkj5c9F14ge5R8HuI2i07hMXXk.IWciB370d2EXwX2').
+
 :- end_tests(crypto_hash).
 
 
